/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2003             *
 * by the XIPHOPHORUS Company http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: PCM data vector blocking, windowing and dis/reassembly
 last mod: $Id: block.c 9513 2005-06-26 18:36:49Z giles $

 Handle windowing, overlap-add, etc of the PCM vectors.  This is made
 more amusing by Vorbis' current two allowed block sizes.
 
 ********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ogg.h"
#include "codec.h"
#include "codec_internal.h"

#include "window.h"
#include "mdct.h"
#include "lpc.h"
#include "registry.h"
#include "misc.h"

extern volatile int heapID;

volatile int memory_req = 0;
volatile int memory_req1 = 0;
volatile void *ext_pointer;
volatile void *int_pointer;

static int ilog2(unsigned int v)
{
  int ret=0;
  if(v)--v;
  while(v)
  {
    ret++;
    v>>=1;
  }
  return(ret);
}

/* pcm accumulator examples (not exhaustive):

 <-------------- lW ---------------->
                   <--------------- W ---------------->
:            .....|.....       _______________         |
:        .'''     |     '''_---      |       |\        |
:.....'''         |_____--- '''......|       | \_______|
:.................|__________________|_______|__|______|
                  |<------ Sl ------>|      > Sr <     |endW
                  |beginSl           |endSl  |  |endSr   
                  |beginW            |endlW  |beginSr


                      |< lW >|       
                   <--------------- W ---------------->
                  |   |  ..  ______________            |
                  |   | '  `/        |     ---_        |
                  |___.'___/`.       |         ---_____| 
                  |_______|__|_______|_________________|
                  |      >|Sl|<      |<------ Sr ----->|endW
                  |       |  |endSl  |beginSr          |endSr
                  |beginW |  |endlW                     
                  mult[0] |beginSl                     mult[n]

 <-------------- lW ----------------->
                          |<--W-->|                               
:            ..............  ___  |   |                    
:        .'''             |`/   \ |   |                       
:.....'''                 |/`....\|...|                    
:.........................|___|___|___|                  
                          |Sl |Sr |endW    
                          |   |   |endSr
                          |   |beginSr
                          |   |endSl
			  |beginSl
			  |beginW
*/

/* block abstraction setup *********************************************/

#ifndef WORD_ALIGN
#define WORD_ALIGN 8
#endif

int vorbis_block_init(vorbis_dsp_state *v, vorbis_block *vb)
{
  int i;
  memset(vb,0,sizeof(*vb));
  vb->vd=v;
  vb->localalloc=0;
  vb->localstore=NULL;

  return(0);
}



void *_vorbis_block_alloc(vorbis_block *vb,long bytes)
{
//heapID = heap_lookup_name("seg_heaq");
	
  bytes=(bytes+(WORD_ALIGN-1)) & ~(WORD_ALIGN-1);
  if(bytes+vb->localtop>vb->localalloc){
    /* can't just _ogg_realloc... there are outstanding pointers */
    if(vb->localstore)
    {
        struct alloc_chain *link;
      /**  if( heapID == 0 )
        {
	   		link=_ogg_malloc(sizeof(*link));
	   		ext_pointer = link;
	   		memory_req1 += sizeof(*link);
        }
	   	else
	   	{ **/
	  		link=_heap_malloc(heapID,sizeof(*link));
	  		memory_req += sizeof(*link);
	  		int_pointer = link;
	   //	}
      vb->totaluse+=vb->localtop;
      link->next=vb->reap;
      link->ptr=vb->localstore;
      vb->reap=link;
    }
    /* highly conservative */
    vb->localalloc=bytes;
/**    if( heapID == 0 )
    {
   		vb->localstore=_ogg_malloc(vb->localalloc);
   		memory_req1 += vb->localalloc;
    }
   	else
   	{ **/
    	vb->localstore=_heap_malloc(heapID,vb->localalloc);
    	memory_req += vb->localalloc;
//	}
    vb->localtop=0;
  }
  {
    void *ret=(void *)(((char *)vb->localstore)+vb->localtop);
    vb->localtop+=bytes;
    return ret;
  }
}

/* reap the chain, pull the ripcord */
void _vorbis_block_ripcord(vorbis_block *vb)
{
  /* reap the chain */
  struct alloc_chain *reap=vb->reap;
  while(reap)
  {
    struct alloc_chain *next=reap->next;
    if( heapID == 0 )
		_ogg_free(reap->ptr);
	else
	{
    	heap_free(heapID,reap->ptr);
	}
    memset(reap,0,sizeof(*reap));
    if( heapID == 0 )
    	_ogg_free(reap);
    else
    {
    	heap_free(heapID,reap);
    }
    reap=next;
  }
  /* consolidate storage */
  if(vb->totaluse)
  {
      if( heapID == 0 )
		vb->localstore=_ogg_realloc(vb->localstore,vb->totaluse+vb->localalloc);
	  else
	  {
		vb->localstore=heap_realloc(heapID,vb->localstore,vb->totaluse+vb->localalloc);
		memory_req += (vb->totaluse+vb->localalloc);
	  }
    vb->localalloc+=vb->totaluse;
    vb->totaluse=0;
  }

  /* pull the ripcord */
  vb->localtop=0;
  vb->reap=NULL;
}

int vorbis_block_clear(vorbis_block *vb)
{
  int i;
  vorbis_block_internal *vbi=vb->internal;

  _vorbis_block_ripcord(vb);
  if(vb->localstore)
  {
  	if( heapID == 0 )
		_ogg_free(vb->localstore);
	else
		heap_free(heapID,vb->localstore);
  } 	
  if(vbi){
    for(i=0;i<PACKETBLOBS;i++){
      oggpack_writeclear(vbi->packetblob[i]);
      if(i!=PACKETBLOBS/2)
      {
          if( heapID == 0 )
      		_ogg_free(vbi->packetblob[i]);
      	  else
      		heap_free(heapID,vbi->packetblob[i]);
      }
    }
    if( heapID == 0 )
   		_ogg_free(vbi);
   	else
   		heap_free(heapID,vbi);
  }
  memset(vb,0,sizeof(*vb));
  return(0);
}

/* Analysis side code, but directly related to blocking.  Thus it's
   here and not in analysis.c (which is for analysis transforms only).
   The init is here because some of it is shared */

static int _vds_shared_init(vorbis_dsp_state *v,vorbis_info *vi,int encp){
  int i;
  codec_setup_info *ci=vi->codec_setup;
  private_state *b=NULL;
  int hs;

  if(ci==NULL) return 1;
  hs=ci->halfrate_flag; 

  memset(v,0,sizeof(*v));
  b=v->backend_state=_ogg_calloc(1,sizeof(*b));

  v->vi=vi;
  b->modebits=ilog2(ci->modes);

  heapID = 1;
  
//  b->transform[0]=_ogg_calloc(VI_TRANSFORMB,sizeof(*b->transform[0]));
//  b->transform[1]=_ogg_calloc(VI_TRANSFORMB,sizeof(*b->transform[1]));

  /* MDCT is tranform 0 */

//  b->transform[0][0]=_ogg_calloc(1,sizeof(mdct_lookup));
//  b->transform[1][0]=_ogg_calloc(1,sizeof(mdct_lookup));
  
  b->transform[0]= heap_calloc(heapID,VI_TRANSFORMB,sizeof(*b->transform[0]));
  b->transform[1]=heap_calloc(heapID,VI_TRANSFORMB,sizeof(*b->transform[1]));

  /* MDCT is tranform 0 */

  b->transform[0][0]= heap_calloc(heapID,1,sizeof(mdct_lookup));
  b->transform[1][0]= heap_calloc(heapID,1,sizeof(mdct_lookup));
  
  mdct_init(b->transform[0][0],ci->blocksizes[0]>>hs);
  mdct_init(b->transform[1][0],ci->blocksizes[1]>>hs);
  heapID = 0;

  /* Vorbis I uses only window type 0 */
  b->window[0]=ilog2(ci->blocksizes[0])-6;
  b->window[1]=ilog2(ci->blocksizes[1])-6;

  /* finish the codebooks */
  if(!ci->fullbooks)
  {
      ci->fullbooks=_ogg_calloc(ci->books,sizeof(*ci->fullbooks));
      for(i=0;i<ci->books;i++)
      {
		vorbis_book_init_decode(ci->fullbooks+i,ci->book_param[i]);
		/* decode codebooks are now standalone after init */
		vorbis_staticbook_destroy(ci->book_param[i]);
		ci->book_param[i]=NULL;
      }
    }
  

  /* initialize the storage vectors. blocksize[1] is small for encode,
     but the correct size for decode */
  v->pcm_storage=ci->blocksizes[1];
  v->pcm=_ogg_malloc(vi->channels*sizeof(*v->pcm));
  v->pcmret=_ogg_malloc(vi->channels*sizeof(*v->pcmret));
  {
    int i;
    for(i=0;i<vi->channels;i++)
      v->pcm[i]=_ogg_calloc(v->pcm_storage,sizeof(*v->pcm[i]));
  } 

  /* all 1 (large block) or 0 (small block) */
  /* explicitly set for the sake of clarity */
  v->lW=0; /* previous window size */
  v->W=0;  /* current window size */

  /* all vector indexes */
  v->centerW=ci->blocksizes[1]/2;

  v->pcm_current=v->centerW;

  /* initialize all the backend lookups */
  b->flr=_ogg_calloc(ci->floors,sizeof(*b->flr));
  b->residue=_ogg_calloc(ci->residues,sizeof(*b->residue));

  for(i=0;i<ci->floors;i++)
    b->flr[i]=_floor_P[ci->floor_type[i]]->
      look(v,ci->floor_param[i]);

  for(i=0;i<ci->residues;i++)
    b->residue[i]=_residue_P[ci->residue_type[i]]->
      look(v,ci->residue_param[i]);    

  return 0;
}

void vorbis_dsp_clear(vorbis_dsp_state *v)
{
  int i;
  if(v)
  {
    vorbis_info *vi=v->vi;
    codec_setup_info *ci=(vi?vi->codec_setup:NULL);
    private_state *b=v->backend_state;

    if(b)
    {
      if(b->transform[0])
      {
		mdct_clear(b->transform[0][0]);
		_ogg_free(b->transform[0][0]);
		_ogg_free(b->transform[0]);
      }
      if(b->transform[1])
      {
		mdct_clear(b->transform[1][0]);
		_ogg_free(b->transform[1][0]);
		_ogg_free(b->transform[1]);
      }

      if(b->flr)
      {
		for(i=0;i<ci->floors;i++)
	  		_floor_P[ci->floor_type[i]]->free_look(b->flr[i]);
		_ogg_free(b->flr);
      }
      if(b->residue)
      {
		for(i=0;i<ci->residues;i++)
	  		_residue_P[ci->residue_type[i]]->free_look(b->residue[i]);
		_ogg_free(b->residue);
      }
    }
    
    if(v->pcm)
    {
      for(i=0;i<vi->channels;i++)
		if(v->pcm[i])
			_ogg_free(v->pcm[i]);
      _ogg_free(v->pcm);
      if(v->pcmret)
      	_ogg_free(v->pcmret);
    }

    if(b)
    {
      /* free header, header1, header2 */
      if(b->header)
      	_ogg_free(b->header);
      if(b->header1)
      	_ogg_free(b->header1);
      if(b->header2)
      	_ogg_free(b->header2);
      _ogg_free(b);
    }
    
    memset(v,0,sizeof(*v));
  }
}

int vorbis_synthesis_restart(vorbis_dsp_state *v)
{
  vorbis_info *vi=v->vi;
  codec_setup_info *ci;
  int hs;

  if(!v->backend_state)
  	return -1;
  if(!vi)
  	return -1;
  ci=vi->codec_setup;
  if(!ci)
  	return -1;
  hs=ci->halfrate_flag; 

  v->centerW=ci->blocksizes[1]>>(hs+1);
  v->pcm_current=v->centerW>>hs;
  
  v->pcm_returned=-1;
  v->granulepos=-1;
  v->sequence=-1;
  v->eofflag=0;
  ((private_state *)(v->backend_state))->sample_count=-1;

  return(0);
}

int vorbis_synthesis_init(vorbis_dsp_state *v,vorbis_info *vi)
{
  if(_vds_shared_init(v,vi,0)) 
  	return 1;
  vorbis_synthesis_restart(v);

  return 0;
}

/* Unlike in analysis, the window is only partially applied for each
   block.  The time domain envelope is not yet handled at the point of
   calling (as it relies on the previous block). */

int vorbis_synthesis_blockin(vorbis_dsp_state *v,vorbis_block *vb)
{
  vorbis_info *vi=v->vi;
  codec_setup_info *ci=vi->codec_setup;
  private_state *b=v->backend_state;
  int hs=ci->halfrate_flag; 
  int i,j;

  if(!vb)
  	return(OV_EINVAL);
  if(v->pcm_current>v->pcm_returned  && v->pcm_returned!=-1)return(OV_EINVAL);
    
  v->lW=v->W;
  v->W=vb->W;
  v->nW=-1;
  
  if((v->sequence==-1)||(v->sequence+1 != vb->sequence))
  {
    v->granulepos=-1; /* out of sequence; lose count */
    b->sample_count=-1;
  }

  v->sequence=vb->sequence;
  
  if(vb->pcm)
  {  /* no pcm to process if vorbis_synthesis_trackonly 
		   was called on block */
    int n=ci->blocksizes[v->W]>>(hs+1);
    int n0=ci->blocksizes[0]>>(hs+1);
    int n1=ci->blocksizes[1]>>(hs+1);

    int thisCenter;
    int prevCenter;
    
    v->glue_bits+=vb->glue_bits;
    v->time_bits+=vb->time_bits;
    v->floor_bits+=vb->floor_bits;
    v->res_bits+=vb->res_bits;
    
    if(v->centerW)
    {
      thisCenter=n1;
      prevCenter=0;
    }
    else
    {
      thisCenter=0;
      prevCenter=n1;
    }
    
    /* v->pcm is now used like a two-stage double buffer.  We don't want
       to have to constantly shift *or* adjust memory usage.  Don't
       accept a new block until the old is shifted out */
    
    for(j=0;j<vi->channels;j++)
    {
      /* the overlap/add section */
      if(v->lW)
      {
		if(v->W)
		{
	  		/* large/large */
	  		float pm *w=(float pm *)_vorbis_window_get(b->window[1]-hs);
	  		float *pcm=v->pcm[j]+prevCenter;
	  		float *p=vb->pcm[j];
	  		for(i=0;i<n1;i++)
	    		pcm[i]=pcm[i]*w[n1-i-1] + p[i]*w[i];
		}
		else
		{
	  		/* large/small */
	  		float pm *w=(float pm *)_vorbis_window_get(b->window[0]-hs);
	  		float *pcm=v->pcm[j]+prevCenter+n1/2-n0/2;
	  		float *p=vb->pcm[j];
	  		for(i=0;i<n0;i++)
	    	pcm[i]=pcm[i]*w[n0-i-1] +p[i]*w[i];
		}
      }
      else
      {
		if(v->W)
		{
	  		/* small/large */
	  		float pm *w=(float pm *)_vorbis_window_get(b->window[0]-hs);
	  		float *pcm=v->pcm[j]+prevCenter;
	  		float *p=vb->pcm[j]+n1/2-n0/2;
	  		for(i=0;i<n0;i++)
	    		pcm[i]=pcm[i]*w[n0-i-1] +p[i]*w[i];
	  		for(;i<n1/2+n0/2;i++)
	    		pcm[i]=p[i];
		}
		else
		{
	  		/* small/small */
	  		float pm *w=(float pm *)_vorbis_window_get(b->window[0]-hs);
	  		float *pcm=v->pcm[j]+prevCenter;
	  		float *p=vb->pcm[j];
	  		for(i=0;i<n0;i++)
	    		pcm[i]=pcm[i]*w[n0-i-1] +p[i]*w[i];
		}
      }
      
      /* the copy section */
      {
		float *pcm=v->pcm[j]+thisCenter;
		float *p=vb->pcm[j]+n;
		
//		pcm_buf = pcm;
		
		for(i=0;i<n;i++)
		{
	  		pcm[i]=p[i];
//	  		final_result = pcm[i]*32767.f;
		}
      }
    }
    
    if(v->centerW)
      v->centerW=0;
    else
      v->centerW=n1;
    
    /* deal with initial packet state; we do this using the explicit
       pcm_returned==-1 flag otherwise we're sensitive to first block
       being short or long */
    
    if(v->pcm_returned==-1)
    {
      v->pcm_returned=thisCenter;
      v->pcm_current=thisCenter;
    }
    else
    {
      v->pcm_returned=prevCenter;
      v->pcm_current=prevCenter+
	((ci->blocksizes[v->lW]/4+
	ci->blocksizes[v->W]/4)>>hs);
    }
    
  }

  /* track the frame number... This is for convenience, but also
     making sure our last packet doesn't end with added padding.  If
     the last packet is partial, the number of samples we'll have to
     return will be past the vb->granulepos.
     
     This is not foolproof!  It will be confused if we begin
     decoding at the last page after a seek or hole.  In that case,
     we don't have a starting point to judge where the last frame
     is.  For this reason, vorbisfile will always try to make sure
     it reads the last two marked pages in proper sequence */

  if(b->sample_count==-1)
  {
    b->sample_count=0;
  }
  else
  {
    b->sample_count+=ci->blocksizes[v->lW]/4+ci->blocksizes[v->W]/4;
  }
  
  if(v->granulepos==-1)
  {
    if(vb->granulepos!=-1)
    { /* only set if we have a position to set to */

      v->granulepos=vb->granulepos;

      /* is this a short page? */
      if(b->sample_count>v->granulepos)
      {
		/* corner case; if this is both the first and last audio page,
	   	then spec says the end is cut, not beginning */
		if(vb->eofflag)
		{
	  	/* trim the end */
	  	/* no preceeding granulepos; assume we started at zero (we'd
	     have to in a short single-page stream) */
	  	/* granulepos could be -1 due to a seek, but that would result
	      in a long count, not short count */
	  
	  		v->pcm_current-=(b->sample_count-v->granulepos)>>hs;
		}
		else
		{
	  	/* trim the beginning */
	  		v->pcm_returned+=(b->sample_count-v->granulepos)>>hs;
	  		if(v->pcm_returned>v->pcm_current)
	    		v->pcm_returned=v->pcm_current;
		}

      }

    }
  }
  else
  {
    v->granulepos+=ci->blocksizes[v->lW]/4+ci->blocksizes[v->W]/4;
    if(vb->granulepos!=-1 && v->granulepos!=vb->granulepos)
    {
      if(v->granulepos>vb->granulepos)
      {
		long extra=v->granulepos-vb->granulepos;

		if(extra)
	  		if(vb->eofflag)
	  		{
	    		/* partial last frame.  Strip the extra samples off */
	    		v->pcm_current-=extra>>hs;
	  		} /* else {Shouldn't happen *unless* the bitstream is out of
	       	spec.  Either way, believe the bitstream } */
      } 	/* else {Shouldn't happen *unless* the bitstream is out of
	   spec.  Either way, believe the bitstream } */
      v->granulepos=vb->granulepos;
    }
  }
  
  /* Update, cleanup */
  
  if(vb->eofflag)
  	v->eofflag=1;
  return(0);
  
}

/* pcm==NULL indicates we just want the pending samples, no more */
int vorbis_synthesis_pcmout(vorbis_dsp_state *v,float ***pcm)
{
  vorbis_info *vi=v->vi;

  if(v->pcm_returned>-1 && v->pcm_returned<v->pcm_current)
  {
    if(pcm)
    {
      int i;
      for(i=0;i<vi->channels;i++)
		v->pcmret[i]=v->pcm[i]+v->pcm_returned;
      *pcm=v->pcmret;
    }
    return(v->pcm_current-v->pcm_returned);
  }
  return(0);
}

int vorbis_synthesis_read(vorbis_dsp_state *v,int n)
{
  if(n && v->pcm_returned+n>v->pcm_current)
  	return(OV_EINVAL);
  v->pcm_returned+=n;
  return(0);
}



